@@ -39,6 +39,7 @@ from _ped import PartitionException
 from _ped import TimerException
 from _ped import UnknownDeviceException
 from _ped import UnknownTypeException
+from _ped import PartedAskException
 
 from alignment import Alignment
 from constraint import Constraint
@@ -105,6 +106,7 @@ from _ped import PARTITION_PREP
 from _ped import PARTITION_MSFT_RESERVED
 from _ped import PARTITION_APPLE_TV_RECOVERY
 from _ped import PARTITION_BIOS_GRUB
+from _ped import PARTITION_DIAG
 
 from _ped import DISK_CYLINDER_ALIGNMENT
 
